# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './nezzle/ui/ui_exportimagedialog.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ExportImageDialog(object):
    def setupUi(self, ExportImageDialog):
        ExportImageDialog.setObjectName("ExportImageDialog")
        ExportImageDialog.resize(344, 406)
        ExportImageDialog.setSizeGripEnabled(False)
        ExportImageDialog.setModal(True)
        self.layoutWidget = QtWidgets.QWidget(ExportImageDialog)
        self.layoutWidget.setGeometry(QtCore.QRect(0, 0, 2, 2))
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.layoutWidget1 = QtWidgets.QWidget(ExportImageDialog)
        self.layoutWidget1.setGeometry(QtCore.QRect(0, 0, 2, 2))
        self.layoutWidget1.setObjectName("layoutWidget1")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.layoutWidget1)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.layoutWidget2 = QtWidgets.QWidget(ExportImageDialog)
        self.layoutWidget2.setGeometry(QtCore.QRect(0, 0, 2, 2))
        self.layoutWidget2.setObjectName("layoutWidget2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.layoutWidget2)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.layoutWidget3 = QtWidgets.QWidget(ExportImageDialog)
        self.layoutWidget3.setGeometry(QtCore.QRect(0, 0, 2, 2))
        self.layoutWidget3.setObjectName("layoutWidget3")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.layoutWidget3)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.layoutWidget4 = QtWidgets.QWidget(ExportImageDialog)
        self.layoutWidget4.setGeometry(QtCore.QRect(12, 8, 321, 388))
        self.layoutWidget4.setObjectName("layoutWidget4")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.layoutWidget4)
        self.verticalLayout_13.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(5, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.label = QtWidgets.QLabel(self.layoutWidget4)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.ui_filePathEdit = QtWidgets.QLineEdit(self.layoutWidget4)
        self.ui_filePathEdit.setReadOnly(False)
        self.ui_filePathEdit.setObjectName("ui_filePathEdit")
        self.horizontalLayout.addWidget(self.ui_filePathEdit)
        self.ui_openButton = QtWidgets.QPushButton(self.layoutWidget4)
        self.ui_openButton.setObjectName("ui_openButton")
        self.horizontalLayout.addWidget(self.ui_openButton)
        spacerItem1 = QtWidgets.QSpacerItem(5, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout_13.addLayout(self.horizontalLayout)
        self.groupBox_5 = QtWidgets.QGroupBox(self.layoutWidget4)
        self.groupBox_5.setFlat(False)
        self.groupBox_5.setCheckable(False)
        self.groupBox_5.setObjectName("groupBox_5")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout(self.groupBox_5)
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.ui_transparencyCheck = QtWidgets.QCheckBox(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ui_transparencyCheck.sizePolicy().hasHeightForWidth())
        self.ui_transparencyCheck.setSizePolicy(sizePolicy)
        self.ui_transparencyCheck.setObjectName("ui_transparencyCheck")
        self.horizontalLayout_5.addWidget(self.ui_transparencyCheck)
        spacerItem2 = QtWidgets.QSpacerItem(5, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem2)
        self.horizontalLayout_13.addLayout(self.horizontalLayout_5)
        self.verticalLayout_13.addWidget(self.groupBox_5)
        spacerItem3 = QtWidgets.QSpacerItem(20, 2, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_13.addItem(spacerItem3)
        self.groupBox_4 = QtWidgets.QGroupBox(self.layoutWidget4)
        self.groupBox_4.setFlat(False)
        self.groupBox_4.setCheckable(False)
        self.groupBox_4.setObjectName("groupBox_4")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.groupBox_4)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.ui_qualitySlider = QtWidgets.QSlider(self.groupBox_4)
        self.ui_qualitySlider.setOrientation(QtCore.Qt.Horizontal)
        self.ui_qualitySlider.setObjectName("ui_qualitySlider")
        self.horizontalLayout_12.addWidget(self.ui_qualitySlider)
        self.ui_qualityEdit = QtWidgets.QLineEdit(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ui_qualityEdit.sizePolicy().hasHeightForWidth())
        self.ui_qualityEdit.setSizePolicy(sizePolicy)
        self.ui_qualityEdit.setMaximumSize(QtCore.QSize(30, 16777215))
        self.ui_qualityEdit.setText("")
        self.ui_qualityEdit.setDragEnabled(True)
        self.ui_qualityEdit.setObjectName("ui_qualityEdit")
        self.horizontalLayout_12.addWidget(self.ui_qualityEdit)
        self.ui_qualityPercentLabel = QtWidgets.QLabel(self.groupBox_4)
        self.ui_qualityPercentLabel.setObjectName("ui_qualityPercentLabel")
        self.horizontalLayout_12.addWidget(self.ui_qualityPercentLabel)
        self.horizontalLayout_4.addLayout(self.horizontalLayout_12)
        self.verticalLayout_13.addWidget(self.groupBox_4)
        spacerItem4 = QtWidgets.QSpacerItem(20, 3, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_13.addItem(spacerItem4)
        self.groupBox_3 = QtWidgets.QGroupBox(self.layoutWidget4)
        self.groupBox_3.setFlat(False)
        self.groupBox_3.setCheckable(False)
        self.groupBox_3.setObjectName("groupBox_3")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.groupBox_3)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.ui_scaleWidthLabel = QtWidgets.QLabel(self.groupBox_3)
        self.ui_scaleWidthLabel.setObjectName("ui_scaleWidthLabel")
        self.horizontalLayout_10.addWidget(self.ui_scaleWidthLabel)
        self.ui_scaleWidthEdit = QtWidgets.QLineEdit(self.groupBox_3)
        self.ui_scaleWidthEdit.setDragEnabled(True)
        self.ui_scaleWidthEdit.setObjectName("ui_scaleWidthEdit")
        self.horizontalLayout_10.addWidget(self.ui_scaleWidthEdit)
        self.horizontalLayout_9.addLayout(self.horizontalLayout_10)
        self.ui_scaleWidthPercentLabel = QtWidgets.QLabel(self.groupBox_3)
        self.ui_scaleWidthPercentLabel.setObjectName("ui_scaleWidthPercentLabel")
        self.horizontalLayout_9.addWidget(self.ui_scaleWidthPercentLabel)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        spacerItem5 = QtWidgets.QSpacerItem(5, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(spacerItem5)
        self.ui_scaleHeightLabel = QtWidgets.QLabel(self.groupBox_3)
        self.ui_scaleHeightLabel.setObjectName("ui_scaleHeightLabel")
        self.horizontalLayout_11.addWidget(self.ui_scaleHeightLabel)
        self.ui_scaleHeightEdit = QtWidgets.QLineEdit(self.groupBox_3)
        self.ui_scaleHeightEdit.setDragEnabled(True)
        self.ui_scaleHeightEdit.setObjectName("ui_scaleHeightEdit")
        self.horizontalLayout_11.addWidget(self.ui_scaleHeightEdit)
        self.horizontalLayout_9.addLayout(self.horizontalLayout_11)
        self.ui_scaleHeightPercentLabel = QtWidgets.QLabel(self.groupBox_3)
        self.ui_scaleHeightPercentLabel.setObjectName("ui_scaleHeightPercentLabel")
        self.horizontalLayout_9.addWidget(self.ui_scaleHeightPercentLabel)
        self.horizontalLayout_3.addLayout(self.horizontalLayout_9)
        self.verticalLayout_13.addWidget(self.groupBox_3)
        spacerItem6 = QtWidgets.QSpacerItem(20, 3, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_13.addItem(spacerItem6)
        self.groupBox_2 = QtWidgets.QGroupBox(self.layoutWidget4)
        self.groupBox_2.setFlat(False)
        self.groupBox_2.setCheckable(False)
        self.groupBox_2.setObjectName("groupBox_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.groupBox_2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_6 = QtWidgets.QLabel(self.groupBox_2)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_6.addWidget(self.label_6)
        self.ui_dpiWidthEdit = QtWidgets.QLineEdit(self.groupBox_2)
        self.ui_dpiWidthEdit.setDragEnabled(True)
        self.ui_dpiWidthEdit.setObjectName("ui_dpiWidthEdit")
        self.horizontalLayout_6.addWidget(self.ui_dpiWidthEdit)
        self.ui_dpiWidthLabel = QtWidgets.QLabel(self.groupBox_2)
        self.ui_dpiWidthLabel.setObjectName("ui_dpiWidthLabel")
        self.horizontalLayout_6.addWidget(self.ui_dpiWidthLabel)
        self.horizontalLayout_8.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        spacerItem7 = QtWidgets.QSpacerItem(5, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem7)
        self.label_7 = QtWidgets.QLabel(self.groupBox_2)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_7.addWidget(self.label_7)
        self.ui_dpiHeightEdit = QtWidgets.QLineEdit(self.groupBox_2)
        self.ui_dpiHeightEdit.setDragEnabled(True)
        self.ui_dpiHeightEdit.setObjectName("ui_dpiHeightEdit")
        self.horizontalLayout_7.addWidget(self.ui_dpiHeightEdit)
        self.ui_dpiHeightLabel = QtWidgets.QLabel(self.groupBox_2)
        self.ui_dpiHeightLabel.setObjectName("ui_dpiHeightLabel")
        self.horizontalLayout_7.addWidget(self.ui_dpiHeightLabel)
        self.horizontalLayout_8.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_2.addLayout(self.horizontalLayout_8)
        self.verticalLayout_13.addWidget(self.groupBox_2)
        spacerItem8 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_13.addItem(spacerItem8)
        self.ui_buttonBox = QtWidgets.QDialogButtonBox(self.layoutWidget4)
        self.ui_buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.ui_buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.ui_buttonBox.setObjectName("ui_buttonBox")
        self.verticalLayout_13.addWidget(self.ui_buttonBox)

        self.retranslateUi(ExportImageDialog)
        self.ui_buttonBox.accepted.connect(ExportImageDialog.accept)
        self.ui_buttonBox.rejected.connect(ExportImageDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(ExportImageDialog)

    def retranslateUi(self, ExportImageDialog):
        _translate = QtCore.QCoreApplication.translate
        ExportImageDialog.setWindowTitle(_translate("ExportImageDialog", "Dialog"))
        self.label.setText(_translate("ExportImageDialog", "File"))
        self.ui_openButton.setText(_translate("ExportImageDialog", "Open"))
        self.groupBox_5.setTitle(_translate("ExportImageDialog", "Background"))
        self.ui_transparencyCheck.setText(_translate("ExportImageDialog", "Transparent Background"))
        self.groupBox_4.setToolTip(_translate("ExportImageDialog", "100% means the image is not compressed."))
        self.groupBox_4.setTitle(_translate("ExportImageDialog", "Quality"))
        self.ui_qualityPercentLabel.setText(_translate("ExportImageDialog", "%"))
        self.groupBox_3.setTitle(_translate("ExportImageDialog", "Scale"))
        self.ui_scaleWidthLabel.setText(_translate("ExportImageDialog", "Width:"))
        self.ui_scaleWidthPercentLabel.setText(_translate("ExportImageDialog", "%"))
        self.ui_scaleHeightLabel.setText(_translate("ExportImageDialog", "Height:"))
        self.ui_scaleHeightPercentLabel.setText(_translate("ExportImageDialog", "%"))
        self.groupBox_2.setTitle(_translate("ExportImageDialog", "DPI"))
        self.label_6.setText(_translate("ExportImageDialog", "Width:"))
        self.ui_dpiWidthLabel.setText(_translate("ExportImageDialog", "dpi"))
        self.label_7.setText(_translate("ExportImageDialog", "Height:"))
        self.ui_dpiHeightLabel.setText(_translate("ExportImageDialog", "dpi"))

