# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './nezzle/ui/ui_mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1347, 833)
        MainWindow.setDockNestingEnabled(True)
        self.ui_centralWidget = QtWidgets.QWidget(MainWindow)
        self.ui_centralWidget.setObjectName("ui_centralWidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.ui_centralWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.ui_mainFrame = QtWidgets.QFrame(self.ui_centralWidget)
        self.ui_mainFrame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.ui_mainFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.ui_mainFrame.setObjectName("ui_mainFrame")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.ui_mainFrame)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.ui_mainTabWidget = QtWidgets.QTabWidget(self.ui_mainFrame)
        self.ui_mainTabWidget.setMinimumSize(QtCore.QSize(0, 0))
        self.ui_mainTabWidget.setBaseSize(QtCore.QSize(0, 0))
        self.ui_mainTabWidget.setTabsClosable(False)
        self.ui_mainTabWidget.setMovable(True)
        self.ui_mainTabWidget.setObjectName("ui_mainTabWidget")
        self.verticalLayout_5.addWidget(self.ui_mainTabWidget)
        self.verticalLayout_2.addWidget(self.ui_mainFrame)
        MainWindow.setCentralWidget(self.ui_centralWidget)
        self.ui_menubar = QtWidgets.QMenuBar(MainWindow)
        self.ui_menubar.setGeometry(QtCore.QRect(0, 0, 1347, 36))
        self.ui_menubar.setObjectName("ui_menubar")
        self.ui_menuFile = QtWidgets.QMenu(self.ui_menubar)
        self.ui_menuFile.setObjectName("ui_menuFile")
        self.ui_menuView = QtWidgets.QMenu(self.ui_menubar)
        self.ui_menuView.setObjectName("ui_menuView")
        self.ui_menuEdit = QtWidgets.QMenu(self.ui_menubar)
        self.ui_menuEdit.setObjectName("ui_menuEdit")
        self.ui_menuGraphics = QtWidgets.QMenu(self.ui_menubar)
        self.ui_menuGraphics.setObjectName("ui_menuGraphics")
        self.ui_menuLock = QtWidgets.QMenu(self.ui_menuGraphics)
        self.ui_menuLock.setObjectName("ui_menuLock")
        self.ui_menuConvert = QtWidgets.QMenu(self.ui_menuGraphics)
        self.ui_menuConvert.setObjectName("ui_menuConvert")
        self.ui_menuLayout = QtWidgets.QMenu(self.ui_menubar)
        self.ui_menuLayout.setObjectName("ui_menuLayout")
        self.ui_menuAlign = QtWidgets.QMenu(self.ui_menuLayout)
        self.ui_menuAlign.setObjectName("ui_menuAlign")
        MainWindow.setMenuBar(self.ui_menubar)
        self.ui_statusbar = QtWidgets.QStatusBar(MainWindow)
        self.ui_statusbar.setObjectName("ui_statusbar")
        MainWindow.setStatusBar(self.ui_statusbar)
        self.ui_navigationDock = DockWidget(MainWindow)
        self.ui_navigationDock.setStyleSheet("")
        self.ui_navigationDock.setObjectName("ui_navigationDock")
        self.ui_controlDockContents = QtWidgets.QWidget()
        self.ui_controlDockContents.setObjectName("ui_controlDockContents")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.ui_controlDockContents)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.ui_groupNavigation = QtWidgets.QGroupBox(self.ui_controlDockContents)
        self.ui_groupNavigation.setObjectName("ui_groupNavigation")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.ui_groupNavigation)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.ui_navigationTree = NetworkTreeView(self.ui_groupNavigation)
        self.ui_navigationTree.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.ui_navigationTree.setObjectName("ui_navigationTree")
        self.verticalLayout_4.addWidget(self.ui_navigationTree)
        self.verticalLayout_3.addWidget(self.ui_groupNavigation)
        self.ui_groupCode = QtWidgets.QGroupBox(self.ui_controlDockContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ui_groupCode.sizePolicy().hasHeightForWidth())
        self.ui_groupCode.setSizePolicy(sizePolicy)
        self.ui_groupCode.setMaximumSize(QtCore.QSize(16777215, 130))
        self.ui_groupCode.setObjectName("ui_groupCode")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.ui_groupCode)
        self.verticalLayout_6.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.widget = QtWidgets.QWidget(self.ui_groupCode)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget.sizePolicy().hasHeightForWidth())
        self.widget.setSizePolicy(sizePolicy)
        self.widget.setObjectName("widget")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout_9.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_8.setContentsMargins(-1, 0, -1, -1)
        self.verticalLayout_8.setSpacing(10)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(self.widget)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.ui_codeFilePathEdit = QtWidgets.QLineEdit(self.widget)
        self.ui_codeFilePathEdit.setObjectName("ui_codeFilePathEdit")
        self.horizontalLayout_2.addWidget(self.ui_codeFilePathEdit)
        self.verticalLayout_8.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem = QtWidgets.QSpacerItem(5, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.ui_openCodeButton = QtWidgets.QPushButton(self.widget)
        self.ui_openCodeButton.setObjectName("ui_openCodeButton")
        self.horizontalLayout_3.addWidget(self.ui_openCodeButton)
        spacerItem1 = QtWidgets.QSpacerItem(5, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.ui_runButton = QtWidgets.QPushButton(self.widget)
        self.ui_runButton.setObjectName("ui_runButton")
        self.horizontalLayout_3.addWidget(self.ui_runButton)
        spacerItem2 = QtWidgets.QSpacerItem(2, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.verticalLayout_8.addLayout(self.horizontalLayout_3)
        self.verticalLayout_9.addLayout(self.verticalLayout_8)
        self.verticalLayout_6.addWidget(self.widget)
        self.verticalLayout_3.addWidget(self.ui_groupCode)
        self.ui_navigationDock.setWidget(self.ui_controlDockContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.ui_navigationDock)
        self.ui_consoleDock = DockWidget(MainWindow)
        self.ui_consoleDock.setFeatures(QtWidgets.QDockWidget.AllDockWidgetFeatures)
        self.ui_consoleDock.setObjectName("ui_consoleDock")
        self.ui_consoleDockConetents = QtWidgets.QWidget()
        self.ui_consoleDockConetents.setObjectName("ui_consoleDockConetents")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.ui_consoleDockConetents)
        self.verticalLayout_10.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.ui_consoleTabWidget = QtWidgets.QTabWidget(self.ui_consoleDockConetents)
        self.ui_consoleTabWidget.setObjectName("ui_consoleTabWidget")
        self.verticalLayout_10.addWidget(self.ui_consoleTabWidget)
        self.ui_consoleDock.setWidget(self.ui_consoleDockConetents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.ui_consoleDock)
        self.ui_visStyleDock = QtWidgets.QDockWidget(MainWindow)
        self.ui_visStyleDock.setEnabled(False)
        self.ui_visStyleDock.setObjectName("ui_visStyleDock")
        self.ui_visStyleDockLayout = QtWidgets.QWidget()
        self.ui_visStyleDockLayout.setObjectName("ui_visStyleDockLayout")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.ui_visStyleDockLayout)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.ui_styleTabWidget = QtWidgets.QTabWidget(self.ui_visStyleDockLayout)
        self.ui_styleTabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.ui_styleTabWidget.setMovable(True)
        self.ui_styleTabWidget.setObjectName("ui_styleTabWidget")
        self.ui_nodeTabLayout = QtWidgets.QWidget()
        self.ui_nodeTabLayout.setObjectName("ui_nodeTabLayout")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.ui_nodeTabLayout)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.ui_nodeStyleTable = QtWidgets.QTableView(self.ui_nodeTabLayout)
        self.ui_nodeStyleTable.setEnabled(False)
        self.ui_nodeStyleTable.setObjectName("ui_nodeStyleTable")
        self.horizontalLayout_5.addWidget(self.ui_nodeStyleTable)
        self.ui_styleTabWidget.addTab(self.ui_nodeTabLayout, "")
        self.ui_linkTabLayout = QtWidgets.QWidget()
        self.ui_linkTabLayout.setObjectName("ui_linkTabLayout")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.ui_linkTabLayout)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.ui_linkStyleTable = QtWidgets.QTableView(self.ui_linkTabLayout)
        self.ui_linkStyleTable.setObjectName("ui_linkStyleTable")
        self.horizontalLayout_6.addWidget(self.ui_linkStyleTable)
        self.ui_styleTabWidget.addTab(self.ui_linkTabLayout, "")
        self.horizontalLayout_4.addWidget(self.ui_styleTabWidget)
        self.ui_visStyleDock.setWidget(self.ui_visStyleDockLayout)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.ui_visStyleDock)
        self.ui_historyDock = DockWidget(MainWindow)
        self.ui_historyDock.setFloating(False)
        self.ui_historyDock.setObjectName("ui_historyDock")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.ui_historyView = HistoryView(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ui_historyView.sizePolicy().hasHeightForWidth())
        self.ui_historyView.setSizePolicy(sizePolicy)
        self.ui_historyView.setObjectName("ui_historyView")
        self.verticalLayout.addWidget(self.ui_historyView)
        self.ui_historyDock.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.ui_historyDock)
        self.ui_actionSaveNetwork = QtWidgets.QAction(MainWindow)
        self.ui_actionSaveNetwork.setObjectName("ui_actionSaveNetwork")
        self.ui_actionExit = QtWidgets.QAction(MainWindow)
        self.ui_actionExit.setObjectName("ui_actionExit")
        self.ui_actionOpenNetwork = QtWidgets.QAction(MainWindow)
        self.ui_actionOpenNetwork.setObjectName("ui_actionOpenNetwork")
        self.ui_actionNewView = QtWidgets.QAction(MainWindow)
        self.ui_actionNewView.setObjectName("ui_actionNewView")
        self.ui_actionSaveSimulation = QtWidgets.QAction(MainWindow)
        self.ui_actionSaveSimulation.setObjectName("ui_actionSaveSimulation")
        self.ui_actionOpenSimulation = QtWidgets.QAction(MainWindow)
        self.ui_actionOpenSimulation.setObjectName("ui_actionOpenSimulation")
        self.ui_actionViewNavigationDock = QtWidgets.QAction(MainWindow)
        self.ui_actionViewNavigationDock.setCheckable(True)
        self.ui_actionViewNavigationDock.setChecked(True)
        self.ui_actionViewNavigationDock.setObjectName("ui_actionViewNavigationDock")
        self.ui_actionViewConsoleDock = QtWidgets.QAction(MainWindow)
        self.ui_actionViewConsoleDock.setCheckable(True)
        self.ui_actionViewConsoleDock.setChecked(True)
        self.ui_actionViewConsoleDock.setObjectName("ui_actionViewConsoleDock")
        self.ui_actionViewVisualStyleDock = QtWidgets.QAction(MainWindow)
        self.ui_actionViewVisualStyleDock.setCheckable(True)
        self.ui_actionViewVisualStyleDock.setChecked(True)
        self.ui_actionViewVisualStyleDock.setObjectName("ui_actionViewVisualStyleDock")
        self.ui_actionExportImage = QtWidgets.QAction(MainWindow)
        self.ui_actionExportImage.setObjectName("ui_actionExportImage")
        self.ui_actionCopy = QtWidgets.QAction(MainWindow)
        self.ui_actionCopy.setObjectName("ui_actionCopy")
        self.ui_actionPaste = QtWidgets.QAction(MainWindow)
        self.ui_actionPaste.setObjectName("ui_actionPaste")
        self.actionLock_Nodes = QtWidgets.QAction(MainWindow)
        self.actionLock_Nodes.setObjectName("actionLock_Nodes")
        self.actionLock_Links = QtWidgets.QAction(MainWindow)
        self.actionLock_Links.setObjectName("actionLock_Links")
        self.ui_actionLockNodes = QtWidgets.QAction(MainWindow)
        self.ui_actionLockNodes.setCheckable(True)
        self.ui_actionLockNodes.setObjectName("ui_actionLockNodes")
        self.ui_actionLockLinks = QtWidgets.QAction(MainWindow)
        self.ui_actionLockLinks.setCheckable(True)
        self.ui_actionLockLinks.setObjectName("ui_actionLockLinks")
        self.ui_actionLockLabels = QtWidgets.QAction(MainWindow)
        self.ui_actionLockLabels.setCheckable(True)
        self.ui_actionLockLabels.setObjectName("ui_actionLockLabels")
        self.ui_actionAlignLeft = QtWidgets.QAction(MainWindow)
        self.ui_actionAlignLeft.setObjectName("ui_actionAlignLeft")
        self.ui_actionAlignCenter = QtWidgets.QAction(MainWindow)
        self.ui_actionAlignCenter.setObjectName("ui_actionAlignCenter")
        self.ui_actionAlignRight = QtWidgets.QAction(MainWindow)
        self.ui_actionAlignRight.setObjectName("ui_actionAlignRight")
        self.ui_actionAlignTop = QtWidgets.QAction(MainWindow)
        self.ui_actionAlignTop.setObjectName("ui_actionAlignTop")
        self.ui_actionAlignMiddle = QtWidgets.QAction(MainWindow)
        self.ui_actionAlignMiddle.setObjectName("ui_actionAlignMiddle")
        self.ui_actionAlignBottom = QtWidgets.QAction(MainWindow)
        self.ui_actionAlignBottom.setObjectName("ui_actionAlignBottom")
        self.ui_actionDistributeHorizontally = QtWidgets.QAction(MainWindow)
        self.ui_actionDistributeHorizontally.setObjectName("ui_actionDistributeHorizontally")
        self.ui_actionDistributeVertically = QtWidgets.QAction(MainWindow)
        self.ui_actionDistributeVertically.setObjectName("ui_actionDistributeVertically")
        self.ui_actionSelectAll = QtWidgets.QAction(MainWindow)
        self.ui_actionSelectAll.setObjectName("ui_actionSelectAll")
        self.ui_actionToCurvedLink = QtWidgets.QAction(MainWindow)
        self.ui_actionToCurvedLink.setObjectName("ui_actionToCurvedLink")
        self.ui_actionToStraightLink = QtWidgets.QAction(MainWindow)
        self.ui_actionToStraightLink.setObjectName("ui_actionToStraightLink")
        self.ui_actionToVerticalElbowLink = QtWidgets.QAction(MainWindow)
        self.ui_actionToVerticalElbowLink.setObjectName("ui_actionToVerticalElbowLink")
        self.ui_actionToHorizontalElbowLink = QtWidgets.QAction(MainWindow)
        self.ui_actionToHorizontalElbowLink.setObjectName("ui_actionToHorizontalElbowLink")
        self.ui_actionToEllipseNode = QtWidgets.QAction(MainWindow)
        self.ui_actionToEllipseNode.setObjectName("ui_actionToEllipseNode")
        self.ui_actionToRectangleNode = QtWidgets.QAction(MainWindow)
        self.ui_actionToRectangleNode.setObjectName("ui_actionToRectangleNode")
        self.ui_actionUndo = QtWidgets.QAction(MainWindow)
        self.ui_actionUndo.setShortcutContext(QtCore.Qt.WindowShortcut)
        self.ui_actionUndo.setObjectName("ui_actionUndo")
        self.ui_actionRedo = QtWidgets.QAction(MainWindow)
        self.ui_actionRedo.setObjectName("ui_actionRedo")
        self.ui_actionViewHistoryDock = QtWidgets.QAction(MainWindow)
        self.ui_actionViewHistoryDock.setCheckable(True)
        self.ui_actionViewHistoryDock.setChecked(True)
        self.ui_actionViewHistoryDock.setObjectName("ui_actionViewHistoryDock")
        self.ui_menuFile.addAction(self.ui_actionOpenNetwork)
        self.ui_menuFile.addSeparator()
        self.ui_menuFile.addAction(self.ui_actionSaveNetwork)
        self.ui_menuFile.addSeparator()
        self.ui_menuFile.addAction(self.ui_actionExportImage)
        self.ui_menuFile.addSeparator()
        self.ui_menuFile.addAction(self.ui_actionExit)
        self.ui_menuView.addAction(self.ui_actionViewNavigationDock)
        self.ui_menuView.addAction(self.ui_actionViewConsoleDock)
        self.ui_menuView.addAction(self.ui_actionViewHistoryDock)
        self.ui_menuEdit.addAction(self.ui_actionUndo)
        self.ui_menuEdit.addAction(self.ui_actionRedo)
        self.ui_menuEdit.addSeparator()
        self.ui_menuEdit.addAction(self.ui_actionCopy)
        self.ui_menuEdit.addAction(self.ui_actionPaste)
        self.ui_menuLock.addAction(self.ui_actionLockNodes)
        self.ui_menuLock.addAction(self.ui_actionLockLinks)
        self.ui_menuLock.addAction(self.ui_actionLockLabels)
        self.ui_menuConvert.addAction(self.ui_actionToEllipseNode)
        self.ui_menuConvert.addAction(self.ui_actionToRectangleNode)
        self.ui_menuConvert.addSeparator()
        self.ui_menuConvert.addAction(self.ui_actionToStraightLink)
        self.ui_menuConvert.addAction(self.ui_actionToCurvedLink)
        self.ui_menuConvert.addAction(self.ui_actionToVerticalElbowLink)
        self.ui_menuConvert.addAction(self.ui_actionToHorizontalElbowLink)
        self.ui_menuGraphics.addAction(self.ui_menuLock.menuAction())
        self.ui_menuGraphics.addAction(self.ui_menuConvert.menuAction())
        self.ui_menuGraphics.addSeparator()
        self.ui_menuGraphics.addAction(self.ui_actionSelectAll)
        self.ui_menuAlign.addAction(self.ui_actionAlignLeft)
        self.ui_menuAlign.addAction(self.ui_actionAlignCenter)
        self.ui_menuAlign.addAction(self.ui_actionAlignRight)
        self.ui_menuAlign.addSeparator()
        self.ui_menuAlign.addAction(self.ui_actionAlignTop)
        self.ui_menuAlign.addAction(self.ui_actionAlignMiddle)
        self.ui_menuAlign.addAction(self.ui_actionAlignBottom)
        self.ui_menuAlign.addSeparator()
        self.ui_menuAlign.addAction(self.ui_actionDistributeHorizontally)
        self.ui_menuAlign.addAction(self.ui_actionDistributeVertically)
        self.ui_menuLayout.addAction(self.ui_menuAlign.menuAction())
        self.ui_menubar.addAction(self.ui_menuFile.menuAction())
        self.ui_menubar.addAction(self.ui_menuEdit.menuAction())
        self.ui_menubar.addAction(self.ui_menuView.menuAction())
        self.ui_menubar.addAction(self.ui_menuGraphics.menuAction())
        self.ui_menubar.addAction(self.ui_menuLayout.menuAction())

        self.retranslateUi(MainWindow)
        self.ui_mainTabWidget.setCurrentIndex(-1)
        self.ui_consoleTabWidget.setCurrentIndex(-1)
        self.ui_styleTabWidget.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.ui_menuFile.setTitle(_translate("MainWindow", "File"))
        self.ui_menuView.setTitle(_translate("MainWindow", "View"))
        self.ui_menuEdit.setTitle(_translate("MainWindow", "Edit"))
        self.ui_menuGraphics.setTitle(_translate("MainWindow", "Graphics"))
        self.ui_menuLock.setTitle(_translate("MainWindow", "Lock"))
        self.ui_menuConvert.setTitle(_translate("MainWindow", "Convert"))
        self.ui_menuLayout.setTitle(_translate("MainWindow", "Layout"))
        self.ui_menuAlign.setTitle(_translate("MainWindow", "Align"))
        self.ui_navigationDock.setWindowTitle(_translate("MainWindow", "Navigation"))
        self.ui_groupNavigation.setTitle(_translate("MainWindow", "Networks"))
        self.ui_groupCode.setTitle(_translate("MainWindow", "From Python Code"))
        self.label.setText(_translate("MainWindow", "File"))
        self.ui_openCodeButton.setText(_translate("MainWindow", "Open"))
        self.ui_runButton.setText(_translate("MainWindow", "Run"))
        self.ui_consoleDock.setWindowTitle(_translate("MainWindow", "Console"))
        self.ui_visStyleDock.setWindowTitle(_translate("MainWindow", "Visual Style"))
        self.ui_styleTabWidget.setTabText(self.ui_styleTabWidget.indexOf(self.ui_nodeTabLayout), _translate("MainWindow", "Node"))
        self.ui_styleTabWidget.setTabText(self.ui_styleTabWidget.indexOf(self.ui_linkTabLayout), _translate("MainWindow", "Link"))
        self.ui_historyDock.setWindowTitle(_translate("MainWindow", "History"))
        self.ui_actionSaveNetwork.setText(_translate("MainWindow", "Save Network"))
        self.ui_actionExit.setText(_translate("MainWindow", "Exit"))
        self.ui_actionExit.setShortcut(_translate("MainWindow", "Ctrl+Q"))
        self.ui_actionOpenNetwork.setText(_translate("MainWindow", "Open Network"))
        self.ui_actionNewView.setText(_translate("MainWindow", "New View"))
        self.ui_actionSaveSimulation.setText(_translate("MainWindow", "Save Simulation"))
        self.ui_actionOpenSimulation.setText(_translate("MainWindow", "Open Simulation"))
        self.ui_actionViewNavigationDock.setText(_translate("MainWindow", "Navigation"))
        self.ui_actionViewConsoleDock.setText(_translate("MainWindow", "Console"))
        self.ui_actionViewVisualStyleDock.setText(_translate("MainWindow", "Visual Style"))
        self.ui_actionExportImage.setText(_translate("MainWindow", "Export Image"))
        self.ui_actionCopy.setText(_translate("MainWindow", "Copy"))
        self.ui_actionPaste.setText(_translate("MainWindow", "Paste"))
        self.actionLock_Nodes.setText(_translate("MainWindow", "Lock Nodes"))
        self.actionLock_Links.setText(_translate("MainWindow", "Lock Links"))
        self.ui_actionLockNodes.setText(_translate("MainWindow", "Lock Nodes"))
        self.ui_actionLockLinks.setText(_translate("MainWindow", "Lock Links"))
        self.ui_actionLockLabels.setText(_translate("MainWindow", "Lock Labels"))
        self.ui_actionAlignLeft.setText(_translate("MainWindow", "Align Left"))
        self.ui_actionAlignCenter.setText(_translate("MainWindow", "Align Center"))
        self.ui_actionAlignRight.setText(_translate("MainWindow", "Align Right"))
        self.ui_actionAlignTop.setText(_translate("MainWindow", "Align Top"))
        self.ui_actionAlignMiddle.setText(_translate("MainWindow", "Align Middle"))
        self.ui_actionAlignBottom.setText(_translate("MainWindow", "Align Bottom"))
        self.ui_actionDistributeHorizontally.setText(_translate("MainWindow", "Distribute Horizontally"))
        self.ui_actionDistributeVertically.setText(_translate("MainWindow", "Distribute Vertically"))
        self.ui_actionSelectAll.setText(_translate("MainWindow", "Select All"))
        self.ui_actionToCurvedLink.setText(_translate("MainWindow", "To Curved Link"))
        self.ui_actionToStraightLink.setText(_translate("MainWindow", "To Straight Link"))
        self.ui_actionToVerticalElbowLink.setText(_translate("MainWindow", "To Vertical Elbow Link"))
        self.ui_actionToHorizontalElbowLink.setText(_translate("MainWindow", "To Horizontal Elbow Link"))
        self.ui_actionToEllipseNode.setText(_translate("MainWindow", "To Ellipse Node"))
        self.ui_actionToRectangleNode.setText(_translate("MainWindow", "To Rectangle Node"))
        self.ui_actionUndo.setText(_translate("MainWindow", "Undo"))
        self.ui_actionRedo.setText(_translate("MainWindow", "Redo"))
        self.ui_actionViewHistoryDock.setText(_translate("MainWindow", "History"))

from nezzle.widgets.dockwidget import DockWidget
from nezzle.widgets.historyview import HistoryView
from nezzle.widgets.networktreeview import NetworkTreeView
