# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './nezzle/ui/ui_opennetworkdialog.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_OpenNetworkDialog(object):
    def setupUi(self, OpenNetworkDialog):
        OpenNetworkDialog.setObjectName("OpenNetworkDialog")
        OpenNetworkDialog.resize(591, 470)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(OpenNetworkDialog.sizePolicy().hasHeightForWidth())
        OpenNetworkDialog.setSizePolicy(sizePolicy)
        OpenNetworkDialog.setWindowOpacity(2.0)
        OpenNetworkDialog.setSizeGripEnabled(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(OpenNetworkDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_fpath = QtWidgets.QLabel(OpenNetworkDialog)
        self.label_fpath.setObjectName("label_fpath")
        self.horizontalLayout.addWidget(self.label_fpath)
        self.ui_filePathEdit = QtWidgets.QLineEdit(OpenNetworkDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ui_filePathEdit.sizePolicy().hasHeightForWidth())
        self.ui_filePathEdit.setSizePolicy(sizePolicy)
        self.ui_filePathEdit.setObjectName("ui_filePathEdit")
        self.horizontalLayout.addWidget(self.ui_filePathEdit)
        self.ui_openButton = QtWidgets.QPushButton(OpenNetworkDialog)
        self.ui_openButton.setObjectName("ui_openButton")
        self.horizontalLayout.addWidget(self.ui_openButton)
        self.ui_reloadButton = QtWidgets.QPushButton(OpenNetworkDialog)
        self.ui_reloadButton.setObjectName("ui_reloadButton")
        self.horizontalLayout.addWidget(self.ui_reloadButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_network_name = QtWidgets.QLabel(OpenNetworkDialog)
        self.label_network_name.setObjectName("label_network_name")
        self.horizontalLayout_2.addWidget(self.label_network_name)
        self.ui_networkNameEdit = QtWidgets.QLineEdit(OpenNetworkDialog)
        self.ui_networkNameEdit.setObjectName("ui_networkNameEdit")
        self.horizontalLayout_2.addWidget(self.ui_networkNameEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.ui_labelLinkHeadMapping = QtWidgets.QLabel(OpenNetworkDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ui_labelLinkHeadMapping.sizePolicy().hasHeightForWidth())
        self.ui_labelLinkHeadMapping.setSizePolicy(sizePolicy)
        self.ui_labelLinkHeadMapping.setObjectName("ui_labelLinkHeadMapping")
        self.verticalLayout.addWidget(self.ui_labelLinkHeadMapping)
        self.ui_linkHeadMappingTable = QtWidgets.QTableWidget(OpenNetworkDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ui_linkHeadMappingTable.sizePolicy().hasHeightForWidth())
        self.ui_linkHeadMappingTable.setSizePolicy(sizePolicy)
        self.ui_linkHeadMappingTable.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustIgnored)
        self.ui_linkHeadMappingTable.setAlternatingRowColors(True)
        self.ui_linkHeadMappingTable.setObjectName("ui_linkHeadMappingTable")
        self.ui_linkHeadMappingTable.setColumnCount(0)
        self.ui_linkHeadMappingTable.setRowCount(0)
        self.ui_linkHeadMappingTable.horizontalHeader().setCascadingSectionResizes(False)
        self.verticalLayout.addWidget(self.ui_linkHeadMappingTable)
        self.ui_labelLinkTailMapping = QtWidgets.QLabel(OpenNetworkDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ui_labelLinkTailMapping.sizePolicy().hasHeightForWidth())
        self.ui_labelLinkTailMapping.setSizePolicy(sizePolicy)
        self.ui_labelLinkTailMapping.setObjectName("ui_labelLinkTailMapping")
        self.verticalLayout.addWidget(self.ui_labelLinkTailMapping)
        self.ui_linkTailMappingTable = QtWidgets.QTableWidget(OpenNetworkDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ui_linkTailMappingTable.sizePolicy().hasHeightForWidth())
        self.ui_linkTailMappingTable.setSizePolicy(sizePolicy)
        self.ui_linkTailMappingTable.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustIgnored)
        self.ui_linkTailMappingTable.setAlternatingRowColors(True)
        self.ui_linkTailMappingTable.setObjectName("ui_linkTailMappingTable")
        self.ui_linkTailMappingTable.setColumnCount(0)
        self.ui_linkTailMappingTable.setRowCount(0)
        self.ui_linkTailMappingTable.horizontalHeader().setCascadingSectionResizes(False)
        self.verticalLayout.addWidget(self.ui_linkTailMappingTable)
        self.ui_buttonBox = QtWidgets.QDialogButtonBox(OpenNetworkDialog)
        self.ui_buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.ui_buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.ui_buttonBox.setObjectName("ui_buttonBox")
        self.verticalLayout.addWidget(self.ui_buttonBox)

        self.retranslateUi(OpenNetworkDialog)
        self.ui_buttonBox.accepted.connect(OpenNetworkDialog.accept)
        self.ui_buttonBox.rejected.connect(OpenNetworkDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(OpenNetworkDialog)

    def retranslateUi(self, OpenNetworkDialog):
        _translate = QtCore.QCoreApplication.translate
        OpenNetworkDialog.setWindowTitle(_translate("OpenNetworkDialog", "Dialog"))
        self.label_fpath.setText(_translate("OpenNetworkDialog", "File Path"))
        self.ui_openButton.setText(_translate("OpenNetworkDialog", "Open"))
        self.ui_reloadButton.setText(_translate("OpenNetworkDialog", "Reload"))
        self.label_network_name.setText(_translate("OpenNetworkDialog", "Network Name"))
        self.ui_labelLinkHeadMapping.setText(_translate("OpenNetworkDialog", "Link Head Mapping"))
        self.ui_linkHeadMappingTable.setSortingEnabled(True)
        self.ui_labelLinkTailMapping.setText(_translate("OpenNetworkDialog", "Link Tail Mapping"))
        self.ui_linkTailMappingTable.setSortingEnabled(True)

